'****************************************************************
'*  Name    : PCF8574A-I2C-2.inc                                *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2015 [AFreixanet 2015]              *
'*          : All Rights Reserved                               *
'*  Date    : 28/06/2023                                        *
'*  Version : 2.0                                               *
'*          : PCF8574A Driver for I2C Software for the PIC18(r) *
'*          : Included I2C Software driver "Software_I2C.Inc"   *
'* Compiler : Free Positron Version 4.0.1.8                     *
'****************************************************************
'
' Version 2.0 28/06/2023.
'/ Write Procedures & new commands

'===============================================================================
'===============================================================================
$ifndef _PCF8574A_
$define _PCF8574A_
'
'$if (_core <> 16)
'    $error "PCA9554A library is only for use with 18F devices (Amicus18 Board)"
'$endif

'/==============================================================================
(*
    I2C-bus to parallel port expander
    400 kHz I2C-bus interface (Fast-mode I2C-bus)
    Operating supply voltage 2.3 V to 5.5 V with 5.5 V tolerant I/Os held to VDD with
    100 uA current source
    8-bit remote I/O pins that default to inputs at power-up
    Latched outputs with 25 mA sink capability for directly driving LEDs
    Total package sink capability of 200 mA
    Active LOW open-drain interrupt output
    Eight programmable slave addresses using three address pins
    Low standby current (4.5 A typical)
    -40 aC to +85 aC operation

Quasi-bidirectional I/Os
    A quasi-bidirectional I/O is an input or output port without using a direction control register.
    Whenever the master reads the register, the value returned to master depends on the
    actual voltage or status of the pin. At power-on, all the ports are HIGH with a weak 100 A
    internal pull-up to VDD, but can be driven LOW by an internal transistor, or an external
    signal. The I/O ports are entirely independent of each other, but each I/O octal is
    controlled by the same read or write data byte.

    Advantages of the quasi-bidirectional I/O over totem pole I/O include:
    -   Better for driving LEDs since the p-channel (transistor to VDD) is small, which saves
        die size and therefore cost. LED drive only requires an internal transistor to ground,
        while the LED is connected to VDD through a current-limiting resistor. Totem pole I/O
        have both n-channel and p-channel transistors, which allow solid HIGH and LOW
        output levels without a pull-up resistor X good for logic levels.
    -   Simpler architecture - only a single register and the I/O can be both input and output
        at the same time. Totem pole I/O have a direction register that specifies the port pin
        direction and it is always in that configuration unless the direction is explicitly
        changed.
    -   Does not require a command byte. The simplicity of one register (no need for the
        pointer register or, technically, the command byte) is an advantage in some
        embedded systems where every byte counts because of memory or bandwidth
        limitations.

    There is only one register to control four possibilities of the port pin: Input HIGH, input
    LOW, output HIGH, or output LOW.
    Input HIGH: The master needs to write 1 to the register to set the port as an input mode
    if the device is not in the default power-on condition. The master reads the register to
    check the input status. If the external source pulls the port pin up to VDD or drives
    logic 1, then the master will read the value of 1.

    Input LOW: The master needs to write 1 to the register to set the port to input mode if
        the device is not in the default power-on condition. The master reads the register to
        check the input status. If the external source pulls the port pin down to VSS or drives
        logic 0, which sinks the weak 100 A current source, then the master will read the value
        of 0.

    Output HIGH: The master writes 1 to the register. There is an additional accelerator or
        strong pull-up current when the master sets the port HIGH. The additional strong pull-up
        is only active during the HIGH time of the acknowledge clock cycle. This accelerator
        current helps the ports 100 A current source make a faster rising edge into a heavily
        loaded output, but only at the start of the acknowledge clock cycle to avoid bus
        contention if an external signal is pulling the port LOW to VSS/driving the port with
        logic 0 at the same time. After the half clock cycle there is only the 100 A current
        source to hold the port HIGH.

    Output LOW: The master writes 0 to the register. There is a strong current sink
        transistor that holds the port pin LOW. A large current may flow into the port, which
        could potentially damage the part if the master writes a 0 to the register and an external
        source is pulling the port HIGH at the same time.

*)
'/==============================================================================
(*
 Proton macros to interface with a Microchip PCF8574A 8-bit I2C I/O peripheral
 using the Software I2C.

 Written for the Free Positron Version 4.0.1.8 using an 18F device

 You are free to use this code as you see fit, however, a mention of the authors
 and compiler would be appreciated.
*)
'-------------------------------------------------------------------------------
'===============================================================================
(*
 This source code is provided 'as is' and any express or implied warranties, including,
 but not limited to, the implied warranties of merchantability and fitness for a particular
 purpose are disclaimed.
  
 This program is free software, you can redistribute it and/or modify.
 The code is offered as-is and the authors take any responsibility for its use.
 This program is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY. However, a mention of the authors and compiler would be appreciated.

 Este programa es un software libre, puede redistribuirlo y/o modificarlo.
 Se entrega este cdigo tal como esta y los autores no se hacen responsable de
 su utilizacin. Este programa se distribuye con la esperanza de que sea til,
 pero SIN NINGUNA GARANTA. No obstante citar los autores y el compilador
 sera muy apreciado.

 Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
 Ce logiciel est offert tel qu'il est, et ses auteurs ne prennent aucune responsabilit
 pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
 mais SANS AUCUNE GARANTIE. Cependant mentionner les auteurs et le compileur serait
 aprci.
*)
'===============================================================================
'===============================================================================  
'/ The PCF8574 library is awailable for use with all PICs devices"
'*******************************************************************************
'* Name    : WriteMSG                                                          *
'* Author  : [Alberto Freixanet]                                               *
'* Purpose : Preprocessor Macro To Write a comment in the Asm file             *
'*******************************************************************************
$ifndef WriteMSG
$define WriteMSG(pMessage) '
Asm                        '
messg pMessage             '
EndAsm

$endif
'/==============================================================================
    Include "GIE_SaveRestore.inc"
'/==============================================================================
$undef _WhatLib
$define _WhatLib "PCF8574-I2C-1.Inc"

$undef _WhatVersion
$define _WhatVersion "Version 1.0 28/06/2023"
'/==============================================================================
'///////////////////////////////////////////////////////////////////////////////
'*******************************************************************************
'/ WARNING: DO NOT MODIFY THE CONFIGURATION REGISTERS => COULD BE A DISASTER!!!
'/ DO NOT COPY SOME CODE FROM THE INCLUDE FILE TO YOUR MAIN PROGRAMM => COULD BE A DISASTER!!!
'/ (ONLY FOR EXPERIMENTED USERS) USE LIBRARY COMMANDS ONLY TO READ/WRITE THE EXPANDER REGISTERS. 
'/ ALL COMMANDS, THAT NOT USED, ARE NOT COMPILED INTO YOUR PIC. (DO NOT WORRY)
'/ THERE ARE A LOT OF CONTROLS, CHECKING THE RETURN ACK, USE THEM TO DEBUG YOUR PROYECT.
'/ IF YOUR PROYECT OVERCOMES THE "PCA9554_Init()" COMMAND THEN => CONGRATULATIONS!!!
'*******************************************************************************
'/////////////////////////////////////////////////////////////////////////////// 

    $ifndef _SDA_Pin
        $define _SDA_Pin PORTC.4    '/ Define the SDA pin for I2C
        $define _AnselSDAPins
    $endif
    $ifndef _SCL_Pin
        $define _SCL_Pin PORTC.3    '/ Define the SCL pin for I2C
        $define _AnselSCLPins
    $endif

    Symbol I2C_SDA_Pin = _SDA_Pin   '/ For I2C software.
    Symbol I2C_SCL_Pin = _SCL_Pin   '/ For I2C software.

    '/ Declare the pin to enable/disable the encoder
    $ifndef EncoderPinEnable
    $define EncoderPinEnable PORTC.5
    $define AnselEncPEnable
    $endif

    '/ Declare the pins to the encoder PORT
    $ifdef EncoderPinEnable
    $define EncoderPORT PORTC
    $define EncoderPin0 PORTC.0
    $define EncoderPin1 PORTC.1    
    $define EncoderPin2 PORTC.2    
    $endif
'/==============================================================================
$ifdef _AnselSDAPins
    $ifdef _ANSELC
        $define SDA_Pin_Ansel ANSELC.4
    $endif
$endif

$ifdef _AnselSCLPins
    $ifdef _ANSELC
        $define SCL_Pin_Ansel ANSELC.3
    $endif
$endif

$ifdef AnselEncPEnable
    $ifdef _ANSELC
        $define EncoderPinEnableAnsel ANSELC.5
    $endif
$endif
'/==============================================================================   
    $ifndef CR
    $define CR 13
    $endif
    
    $ifndef LF
    $define LF 10
    $endif
    
    $ifndef BS
    $define BS 8
    $endif
    
    $ifndef True
    $define True 1
    $endif
    
    $ifndef False
    $define False 0
    $endif

'///////////////////////////////////////////////////////////////////////////////
' Define the Input Address A2,A1,A0 of the PCA9554/PCA9554A

'/ Define number of Expanders connected.
$ifndef PCF8574x_MaxNumber
    $define PCF8574x_MaxNumber True
$endif

$if PCF8574x_MaxNumber > 8
     $SendError "NO such number of expanders!"
$endif
$if PCF8574x_MaxNumber = False
     $SendError "NO such number of expanders!"
$endif

$if _defined(PCF8574)
     $define PCACtrlWrite %01000000
     $define PCACtrlRead  %01000001
$elseif _defined(PCF8574A)
     $define PCACtrlWrite %01110000
     $define PCACtrlRead  %01110001
$else
     $SendError "NO PCF8574x Unit has been declared!"
$endif

$if _defined (PCF8574) And _defined (PCF8574A)
     $SendError "Too much PCF8574x Units have been declared!"
$endif

$define DisableEncoder() High EncoderPinEnable

$define EnableEncoder() Low EncoderPinEnable
'
' Control byte for IIC operations [0100][A2][A1][A0][R=1/W=0] for PCF8574
' Control byte for IIC operations [0111][A2][A1][A0][R=1/W=0] for PCF8574A
'===============================================================================  
' Definition for the global Variables
    Dim nCTL_PCF_Read  As Byte System
    Dim nCTL_PCF_Write As Byte System
    Dim nPCAAddress   As Byte System
    Dim nPCAtempByte1 As Byte System
    Dim nDataBytepcf0 As Byte System
    Dim nDataBytepcf1 As Byte System    
    Dim vPCA_ACK As Byte System

    Dim PCA_Flags As Byte System
    Dim F_NoACK As PCA_Flags.0
    Dim F_SkipOverCode As PCA_Flags.1

    Dim xINTF As Byte System
    Dim xGPIO As Byte System
    Dim yGPIO As Byte System
    Dim xOLAT As Byte System  
    Dim vOLATOld As Byte System

    $if PCF8574x_MaxNumber = 1
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $endif

    $if PCF8574x_MaxNumber = 2
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $endif

    $if PCF8574x_MaxNumber = 3
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $define vGPIO2 vGPIO[2]
    $endif

    $if PCF8574x_MaxNumber = 4
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $define vGPIO2 vGPIO[2]
    $define vGPIO3 vGPIO[3]
    $endif

    $if PCF8574x_MaxNumber = 5
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $define vGPIO2 vGPIO[2]
    $define vGPIO3 vGPIO[3]
    $define vGPIO4 vGPIO[4]
    $endif

    $if PCF8574x_MaxNumber = 6
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $define vGPIO2 vGPIO[2]
    $define vGPIO3 vGPIO[3]
    $define vGPIO4 vGPIO[4]
    $define vGPIO5 vGPIO[5]
    $endif

    $if PCF8574x_MaxNumber = 7
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $define vGPIO2 vGPIO[2]
    $define vGPIO3 vGPIO[3]
    $define vGPIO4 vGPIO[4]
    $define vGPIO5 vGPIO[5]
    $define vGPIO6 vGPIO[6]
    $endif

    $if PCF8574x_MaxNumber = 8
    Dim vGPIO[PCF8574x_MaxNumber] As Byte Heap
    $define vGPIO0 vGPIO[0]
    $define vGPIO1 vGPIO[1]
    $define vGPIO2 vGPIO[2]
    $define vGPIO3 vGPIO[3]
    $define vGPIO4 vGPIO[4]
    $define vGPIO5 vGPIO[5]
    $define vGPIO6 vGPIO[6]
    $define vGPIO7 vGPIO[7]
    $endif
    '/--------------------------------------------------------------------------
    $if PCF8574x_MaxNumber = 1
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $endif

    $if PCF8574x_MaxNumber = 2
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $endif

    $if PCF8574x_MaxNumber = 3
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $define vOLAT2 vOLAT[2]
    $endif

    $if PCF8574x_MaxNumber = 4
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $define vOLAT2 vOLAT[2]
    $define vOLAT3 vOLAT[3]
    $endif

    $if PCF8574x_MaxNumber = 5
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $define vOLAT2 vOLAT[2]
    $define vOLAT3 vOLAT[3]
    $define vOLAT4 vOLAT[4]
    $endif

    $if PCF8574x_MaxNumber = 6
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $define vOLAT2 vOLAT[2]
    $define vOLAT3 vOLAT[3]
    $define vOLAT4 vOLAT[4]
    $define vOLAT5 vOLAT[5]
    $endif

    $if PCF8574x_MaxNumber = 7
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $define vOLAT2 vOLAT[2]
    $define vOLAT3 vOLAT[3]
    $define vOLAT4 vOLAT[4]
    $define vOLAT5 vOLAT[5]
    $define vOLAT6 vOLAT[6]
    $endif

    $if PCF8574x_MaxNumber = 8
    Dim vOLAT[PCF8574x_MaxNumber] As Byte Heap
    $define vOLAT0 vOLAT[0]
    $define vOLAT1 vOLAT[1]
    $define vOLAT2 vOLAT[2]
    $define vOLAT3 vOLAT[3]
    $define vOLAT4 vOLAT[4]
    $define vOLAT5 vOLAT[5]
    $define vOLAT6 vOLAT[6]
    $define vOLAT7 vOLAT[7]
    $endif
    '/--------------------------------------------------------------------------
    $if PCF8574x_MaxNumber = 1
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $endif

    $if PCF8574x_MaxNumber = 2
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $endif

    $if PCF8574x_MaxNumber = 3
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $define vIODIR2 vIODIR[2]
    $endif

    $if PCF8574x_MaxNumber = 4
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $define vIODIR2 vIODIR[2]
    $define vIODIR3 vIODIR[3]
    $endif

    $if PCF8574x_MaxNumber = 5
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $define vIODIR2 vIODIR[2]
    $define vIODIR3 vIODIR[3]
    $define vIODIR4 vIODIR[4]
    $endif

    $if PCF8574x_MaxNumber = 6
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $define vIODIR2 vIODIR[2]
    $define vIODIR3 vIODIR[3]
    $define vIODIR4 vIODIR[4]
    $define vIODIR5 vIODIR[5]
    $endif

    $if PCF8574x_MaxNumber = 7
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $define vIODIR2 vIODIR[2]
    $define vIODIR3 vIODIR[3]
    $define vIODIR4 vIODIR[4]
    $define vIODIR5 vIODIR[5]
    $define vIODIR6 vIODIR[6]
    $endif

    $if PCF8574x_MaxNumber = 8
    Dim vIODIR[PCF8574x_MaxNumber] As Byte Heap
    $define vIODIR0 vIODIR[0]
    $define vIODIR1 vIODIR[1]
    $define vIODIR2 vIODIR[2]
    $define vIODIR3 vIODIR[3]
    $define vIODIR4 vIODIR[4]
    $define vIODIR5 vIODIR[5]
    $define vIODIR6 vIODIR[6]
    $define vIODIR7 vIODIR[7]
    $endif
    '/-------------------------------------------------------------------------- 
    $if PCF8574x_MaxNumber = 1
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $endif

    $if PCF8574x_MaxNumber = 2
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $endif

    $if PCF8574x_MaxNumber = 3
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $define vIPOL2 vIPOL[2]
    $endif

    $if PCF8574x_MaxNumber = 4
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $define vIPOL2 vIPOL[2]
    $define vIPOL3 vIPOL[3]
    $endif

    $if PCF8574x_MaxNumber = 5
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $define vIPOL2 vIPOL[2]
    $define vIPOL3 vIPOL[3]
    $define vIPOL4 vIPOL[4]
    $endif

    $if PCF8574x_MaxNumber = 6
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $define vIPOL2 vIPOL[2]
    $define vIPOL3 vIPOL[3]
    $define vIPOL4 vIPOL[4]
    $define vIPOL5 vIPOL[5]
    $endif

    $if PCF8574x_MaxNumber = 7
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $define vIPOL2 vIPOL[2]
    $define vIPOL3 vIPOL[3]
    $define vIPOL4 vIPOL[4]
    $define vIPOL5 vIPOL[5]
    $define vIPOL6 vIPOL[6]
    $endif

    $if PCF8574x_MaxNumber = 8
    Dim vIPOL[PCF8574x_MaxNumber] As Byte Heap
    $define vIPOL0 vIPOL[0]
    $define vIPOL1 vIPOL[1]
    $define vIPOL2 vIPOL[2]
    $define vIPOL3 vIPOL[3]
    $define vIPOL4 vIPOL[4]
    $define vIPOL5 vIPOL[5]
    $define vIPOL6 vIPOL[6]
    $define vIPOL7 vIPOL[7]
    $endif
    '/--------------------------------------------------------------------------
'///////////////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////   
' Aliases for Interrupts flags
    $define INT0IE INTCONBits_INT0IE ' INT0 External Interrupt Enable Bit.

    $ifdef _INTCON3
        $define INT1IE INTCON3Bits_INT1IE   ' INT1 External Interrupt Enable Bit.
        $define INT2IE INTCON3Bits_INT2IE   ' INT2 External Interrupt Enable Bit.
    $else
        $SendError "There is NOT INTCON3 SFR"
    $endif

    $define INT0IF INTCONBits_INT0IF        ' INT0 External Interrupt Flag bit.
    $define INT1IF INTCON3Bits_INT1IF       ' INT1 External Interrupt Flag bit.
    $define INT2IF INTCON3Bits_INT2IF       ' INT2 External Interrupt Flag bit.

    $ifdef _INTCON2
        $define INTEDG0 INTCON2Bits_INTEDG0 ' External Interrupt 0 Edge Select Bit
        $define INTEDG1 INTCON2Bits_INTEDG1 ' External Interrupt 1 Edge Select Bit
        $define INTEDG2 INTCON2Bits_INTEDG2 ' External Interrupt 2 Edge Select Bit    
    $else
        $SendError "There is NOT INTCON2 SFR"    
    $endif

    $define INT1IP  INTCON3Bits_INT1IP      ' INT1 External Interrupt Priority Bit
    $define INT2IP  INTCON3Bits_INT2IP      ' INT2 External Interrupt Priority Bit  

'=============================================================================== 
' Others library commands 
$define Declare_InputMode8(pALias,pPin) Dim pALias As xINTF.pPin 

$define Declare_OutputExp0(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin 

$define Declare_OutputExp1(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin 

$define Declare_OutputExp2(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin 

$define Declare_OutputExp3(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin

$define Declare_OutputExp4(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin 

$define Declare_OutputExp5(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin 

$define Declare_OutputExp6(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin 

$define Declare_OutputExp7(pALias,pPin) '
Dim pALias As xOLAT.pPin                '
Symbol pALias#Pin = pPin
  
'===============================================================================                  
$define SkipAfterUserCode()   '
If F_SkipOverCode = True Then '
    F_SkipOverCode = False    '
    Continue                  '
EndIf                         '
#define SkipOverCode = True

'=============================================================================== 
' Enable INTx interrupt
' Input     : None
' Output    : None
' Notes     : 
'
$define PCF8574_Disable_INTxIE() '    
    #ifdef __INTERRUPTS_ENABLED  '
    $ifdef _INT0_                '
    INT0IE = False               '
    $endif                       '
    $ifdef _INT1_                '
    INT1IE = False               '
    $endif                       '
    $ifdef _INT2_                '
    INT2IE = False               '
    $endif                       '
    #endif 



$define PCF8574_Enable_INTxIE() '    
    #ifdef __INTERRUPTS_ENABLED '
    $ifdef _INT0_               '
    INT0IF = False              '
    INT0IE = True               '
    $endif                      '
    $ifdef _INT1_               '
    INT1IF = False              '
    INT1IE = True               '
    $endif                      '
    $ifdef _INT2_               '
    INT2IF = False              '
    INT2IE = True               '
    $endif                      '
    #endif 


$define PCF8574_Clear_INTxIF()  '    
    #ifdef __INTERRUPTS_ENABLED '
    $ifdef _INT0_               '
    INT0IF = False              '
    $endif                      '
    $ifdef _INT1_               '
    INT1IF = False              '
    $endif                      '
    $ifdef _INT2_               '
    INT2IF = False              '
    $endif                      '
    #endif


'===============================================================================   
'===============================================================================
' Configure Register by Register the PCA9554
(* IODIR DIRECTION PORT REGISTER (ADDR 0x03) *)
$define P0_Input %11111111
$define P1_Input %11111111
$define P2_Input %11111111
$define P3_Input %11111111
$define P4_Input %11111111
$define P5_Input %11111111
$define P6_Input %11111111
$define P7_Input %11111111
'
$define P0_Output %11111110
$define P1_Output %11111101
$define P2_Output %11111011
$define P3_Output %11110111
$define P4_Output %11101111
$define P5_Output %11011111
$define P6_Output %10111111
$define P7_Output %01111111
'
(* IPOL  Input POLARITY PORT REGISTER (ADDR 0x02) *)
$define P0_Open %11111111
$define P1_Open %11111111
$define P2_Open %11111111
$define P3_Open %11111111
$define P4_Open %11111111
$define P5_Open %11111111
$define P6_Open %11111111
$define P7_Open %11111111
'
$define P0_Closed %11111110
$define P1_Closed %11111101
$define P2_Closed %11111011
$define P3_Closed %11110111
$define P4_Closed %11101111
$define P5_Closed %11011111
$define P6_Closed %10111111
$define P7_Closed %01111111
'
$define P0_None %11111110
$define P1_None %11111101
$define P2_None %11111011
$define P3_None %11110111
$define P4_None %11101111
$define P5_None %11011111
$define P6_None %10111111
$define P7_None %01111111
'
    $if PCF8574x_MaxNumber > 0
    $define Config_IODIR0(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR0_Configured = 1                                                                     '
    vIODIR0 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7)
    
    $endif
    
    $if PCF8574x_MaxNumber > 1
    $define Config_IODIR1(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR1_Configured = 1                                                                     '
    vIODIR1 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7)
    
    $endif
    
    $if PCF8574x_MaxNumber > 2
    $define Config_IODIR2(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR2_Configured = 1                                                                     '
    vIODIR2 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7)
    
    $endif
    
    $if PCF8574x_MaxNumber > 3
    $define Config_IODIR3(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR3_Configured = 1                                                                     '
    vIODIR3 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif
    
    $if PCF8574x_MaxNumber > 4
    $define Config_IODIR4(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR4_Configured = 1                                                                     '
    vIODIR4 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7)
    
    $endif
    
    $if PCF8574x_MaxNumber > 5
    $define Config_IODIR5(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR5_Configured = 1                                                                     '
    vIODIR5 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7)
    
    $endif
    
    $if PCF8574x_MaxNumber > 6
    $define Config_IODIR6(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR6_Configured = 1                                                                     '
    vIODIR6 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif
    
    $if PCF8574x_MaxNumber > 7
    $define Config_IODIR7(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IODIR7_Configured = 1                                                                     '
    vIODIR7 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif
        
'-------------------------------------------------------------------------------
' EXAMBLE: COPY OR MODIFY TO THE BASIC PROGRAM.
(*
    ' IODIR  I/O DIRECTION REGISTER (ADDR 0x00)
    ' P0_Input or P0_Output
    ' P1_Input Or P1_Output
    ' P2_Input Or P2_Output
    ' P3_Input Or P3_Output
    ' P4_Input Or P4_Output
    ' P5_Input Or P5_Output
    ' P6_Input Or P6_Output
    ' P7_Input Or P7_Output
    Config_IODIR(P7_Output, P6_Output, P5_Output, P4_Output, P3_Input, P2_Input, P1_Input, P0_Input)

    ' IPOL  INPUT POLARITY PORT REGISTER (ADDR 0x01)
    ' PX_None for pins configured as Output.
    ' P0_Inverted or P0_NotInverted or P0_None if output
    ' P1_Inverted Or P1_NotInverted Or P1_None If Output 
    ' P2_Inverted Or P2_NotInverted Or P2_None If Output 
    ' P3_Inverted Or P3_NotInverted Or P3_None If Output 
    ' P4_Inverted Or P4_NotInverted Or P4_None If Output 
    ' P5_Inverted Or P5_NotInverted Or P5_None If Output 
    ' P6_Inverted Or P6_NotInverted Or P6_None If Output 
    ' P7_Inverted Or P7_NotInverted Or P7_None If Output 

    '/ Configure the Inputs Polarity
    '/ The inputs are inverted because all buttons are connected to 0V.
    '/ If Buttons are connected to ground then write the configuration => PX_Inverted
    '/ If Buttons are connected to VDD then write the configuration => PX_NotInverted Or PX_None
    Config_IPOL(P7_None, P6_None, P5_None, P4_None, P3_Inverted, P2_Inverted, P1_Inverted, P0_Inverted)
*)
    $if PCF8574x_MaxNumber > 0
    $define Config_IPOL0(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IPOL0_Configured = 1                                                                     '
    vIPOL0 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 1
    $define Config_IPOL1(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)   '
    #define IPOL1_Configured = 1                                                                    '
    vIPOL1 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 2
    $define Config_IPOL2(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)   '
    #define IPOL2_Configured = 1                                                                    '
    vIPOL2 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 3
    $define Config_IPOL3(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IPOL3_Configured = 1                                                                     '
    vIPOL3 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 4
    $define Config_IPOL4(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IPOL4_Configured = 1                                                                     '
    vIPOL4 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 5
    $define Config_IPOL5(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IPOL5_Configured = 1                                                                     '
    vIPOL5 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 6
    $define Config_IPOL6(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IPOL6_Configured = 1                                                                     '
    vIPOL6 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

    $if PCF8574x_MaxNumber > 7
    $define Config_IPOL7(pConfig0,pConfig1,pConfig2,pConfig3,pConfig4,pConfig5,pConfig6,pConfig7)    '
    #define IPOL7_Configured = 1                                                                     '
    vIPOL7 = (pConfig0 & pConfig1 & pConfig2 & pConfig3 & pConfig4 & pConfig5 & pConfig6 & pConfig7) 
    
    $endif

'/------------------------------------------------------------------------------
(*
' Initialize the expander and check the transmision.
    CheckAck = PCA9554_Init()

    ' Control the ACK error of the I2C bus of the PCA9554.
    If CheckAck = $0F Then
        HRSOut "NACK I2C comunication error with the PCA9535 expander",CR
        HRSOut "The PCA9554 is not initialized",CR
        HRSOut "The demo EXITS here",CR
        Stop
    ElseIf CheckAck = $1F Then
        HRSOut "NACK writing configuration error with the PCA9554 expander",CR
        HRSOut "The PCA9554 is not initialized",CR
        HRSOut "The demo EXITS here",CR
        Stop
    ElseIf CheckAck = 0 Then    
        HRSOut "PCA9554 Received ACK OK: CONGRATULATIONS!!!",CR
    EndIf         
    CheckINTxPins()     '/ Show the state of the INTX pins. (not mandatory)
    HRSOut CR
*)
'===============================================================================   
'///////////////////////////////////////////////////////////////////////////////
'/ Create a software I2C Library:
'/ Create global variables used by the routines
    Dim I2C_bTemp As Byte Access
'/==============================================================================
' Delay a fixed number of microseconds (original DelayUS 5)
' Adjust the SCL bus speed: The speed can be adjusted externally.
    $if _Hbus_Bitrate_ = 100
        $define I2C_hDelay() DelayUS 2
    $else
        $ifndef I2C_hDelay
        $define I2C_hDelay() DelayUS 5
        $endif
    $endif
'///////////////////////////////////////////////////////////////////////////////
'/ Shift out a Byte To the I2C bus
'/ Input : pData holds the Byte To write
'/ Output: none
'/ Syntax: MSB first
'/ Notes : WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_hShiftOut(pValueIn As I2C_bTemp) 
    Dim _Index As Byte  '/ Local Byte.
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_hShiftOut : START")
    '/--------------------------------------------------------------------------
    _Index = 8
    Repeat
        If I2C_bTemp.7 = 1 Then
            Input I2C_SDA_Pin
        Else
            Output I2C_SDA_Pin
        EndIf
        I2C_hDelay()
        Input I2C_SCL_Pin
        I2C_hDelay()
        Output I2C_SCL_Pin
        I2C_hDelay()
        I2C_bTemp = I2C_bTemp << 1
        Dec _Index
    Until _Index = 0
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_hShiftOut : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Read a single Byte from the I2C bus     (Proc I2C_hShiftIn(), I2C_bTemp)
'/ Input : none
'/ Output: pResult holds the Byte received
'/ Syntax: MSB first
'/ Notes : MSB first, sample before clock
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_ReadByte(), I2C_bTemp 
    Dim _Index As Byte      '/ Local Byte.
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_ReadByte : START")
    '/--------------------------------------------------------------------------
    I2C_bTemp = 0
    Input I2C_SDA_Pin
    _Index = 8
    Repeat
        I2C_bTemp = I2C_bTemp << 1
        '
        ' Wait for any clock stretching (This may benefit from a timeout being added)
        '
        Repeat
            Input I2C_SCL_Pin
        Until I2C_SCL_Pin = 1
        '
        ' Read the data
        '
        I2C_hDelay()
        I2C_bTemp.0 = I2C_SDA_Pin
        Output I2C_SCL_Pin
        I2C_hDelay()
        Dec _Index
    Until _Index = 0
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_ReadByte : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Write a single Byte To the I2C bus
'/ Input : pData holds the Byte To write
'/ Output: I2C_NACK holds the Acknowledge Return from the bus
'/ Syntax: I2C_WriteByte(ValueOut)
'/ Notes : MSB first
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_WriteByte(pValueOut As I2C_bTemp)
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_WriteByte : START")
    '/--------------------------------------------------------------------------
    I2C_hShiftOut(I2C_bTemp)
    Input I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    F_NoACK = I2C_SDA_Pin   '/ Receive a NACK?
    Output I2C_SCL_Pin
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_WriteByte : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Send an I2C bus start condition
'/ Input : none
'/ Output: none
'/ Syntax: I2C_Start()
'/ Notes : A Start condition is High To Low of SDA when SCL is High
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_Start()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_Start : START")
    '/--------------------------------------------------------------------------
    Input I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SDA_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    I2C_hDelay()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_Start : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Send an I2C bus restart condition
'/ Input : none
'/ Output: none
'/ Syntax: I2C_ReStart()
'/ Notes : Same As Start.
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_ReStart()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_ReStart : START")
    '/--------------------------------------------------------------------------
    Input I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SDA_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    I2C_hDelay()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_ReStart : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Send an I2C bus Stop condition
'/ Input : none
'/ Output: none
'/ Syntax: I2C_Stop()
'/ Notes : A Stop condition is Low To High of SDA when SCL is High
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_Stop()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_Stop : START")
    '/--------------------------------------------------------------------------
    Output I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Input I2C_SDA_Pin
    I2C_hDelay()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_Stop : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Initiate an I2C acknowledge
'/ Input : none
'/ Output: none
'/ Syntax: I2C_Ack()
'/ Notes : none
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_Ack()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_Ack : START")
    '/--------------------------------------------------------------------------
    Output I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    Input I2C_SDA_Pin
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_Ack : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Initiate an I2C Not acknowledge
'/ Input : none
'/ Output: none
'/ Syntax: I2C_NACK()
'/ Notes : none
'/ WARNING: Command For Library ONLY: NOT use in main Code.
'/
Proc I2C_NACK()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_NACK : START")
    '/--------------------------------------------------------------------------
    Input I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    Input I2C_SDA_Pin
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: I2C_NACK : END")
    '/--------------------------------------------------------------------------
EndProc
'*******************************************************************************
'///////////////////////////////////////////////////////////////////////////////
'\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

'///////////////////////////////////////////////////////////////////////////////
'/ Check the PORT of the encoder 74HC148
'/ Input : PORTx
'/ Output: Return Var
'/ Syntax: Return Var = Read_HC148(PORTC)
'/ Notes : 
'/
'$define Read_HC148(pPORT,pResult) mReadPORT_HC148 pPORT

'mReadPORT_HC148 Macro- pPORT\Byte
'    #if (Prm_1 == Byte) 
'        Byte_Wreg pPORT         
'    #endif
'    WREG = ~WREG
'    WREG = WREG & %00000111
'    Return_WREG
'Endm

Proc Read_HC148(pPORT As WREG), WREG
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: Read_HC148 : START")
    '/--------------------------------------------------------------------------
    WREG = ~WREG
    WREG = WREG & %00000111    
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: Read_HC148 : END")
    '/--------------------------------------------------------------------------
EndProc

'///////////////////////////////////////////////////////////////////////////////
'/ Gets Shadow registers And copy To main program
'/ Input : Shadow registers
'/ Output: Return Var
'/ Syntax: ReturnVar = PCF8574_Read_GPIO_Shadow(NumberExpander)
'/ Notes : Reads the shadow register For testing
'/
$define PCF8574_Read_GPIO_Shadow(pNbExp) _Read_GPIO_Shadow pNbExp

_Read_GPIO_Shadow Macro- pNbExp\Byte
    #if (Prm_1 == Byte) 
        Byte_Wreg pNbExp        
    #endif
    WREG = vGPIO[WREG]
    Return_WREG
Endm

'/------------------------------------------------------------------------------
$define PCF8574_Read_IODIR_Shadow(pNbExp) _Read_IODIR_Shadow pNbExp

_Read_IODIR_Shadow Macro- pNbExp\Byte
    #if (Prm_1 == Byte) 
        Byte_Wreg pNbExp        
    #endif
    WREG = vIODIR[WREG]
    Return_WREG
Endm

'/------------------------------------------------------------------------------
$define PCF8574_Read_IPOL_Shadow(pNbExp) _Read_IPOL_Shadow pNbExp

_Read_IPOL_Shadow Macro- pNbExp\Byte
    #if (Prm_1 == Byte) 
        Byte_Wreg pNbExp        
    #endif
    WREG = vIPOL[WREG]
    Return_WREG
Endm

'/------------------------------------------------------------------------------
$define PCF8574_Read_INTF_Shadow() _Read_INTF_Shadow 

_Read_INTF_Shadow Macro-\Byte
    Return_Byte xINTF
Endm

'/------------------------------------------------------------------------------
$define PCF8574_Read_NbExpander() _Read_NbExpander 

_Read_NbExpander Macro-\Byte
    Return_Byte nPCAAddress
Endm

'/------------------------------------------------------------------------------
$define PCF8574_Read_ControlWrite() _PCFControWrite 

_PCFControWrite Macro-\Byte
    Return_Byte nCTL_PCF_Write
Endm

'/------------------------------------------------------------------------------
$define PCF8574_Read_ControlRead() _PCFControRead 

_PCFControRead Macro-\Byte
    Return_Byte nCTL_PCF_Read
Endm

'///////////////////////////////////////////////////////////////////////////////
'/ Check the number of expander & set the Control Bytes of expander.
'/ Input : nPCAAddress
'/ Output: Control Bytes
'/ Syntax: CheckExpander()
'/ Notes : WARNING: Command for Library ONLY: NOT use in main code.
'/  Standart code:
'/  nCTL_PCF_Read  = %01000001 | ((AddressExpander & %00000111) << 1) '/ To read
'/  nCTL_PCF_Write = %01000000 | ((AddressExpander & %00000111) << 1) '/ To write
'/
'/ Procedure & library prepared for 8 expanders. (0 to 7)
'/ Notes : WARNING: This Command is For Library ONLY: DO NOT use in main Code.
'/
Proc CheckExpander()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: CheckExpander : START")
    '/--------------------------------------------------------------------------
    If nPCAAddress > PCF8574x_MaxNumber - 1 Then
        vPCA_ACK = $55
        ExitProc
    EndIf
    nCTL_PCF_Read  = PCACtrlRead  | ((nPCAAddress & %00000111) << 1)
    nCTL_PCF_Write = PCACtrlWrite | ((nPCAAddress & %00000111) << 1)
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: CheckExpander : END")
    '/--------------------------------------------------------------------------
EndProc

'///////////////////////////////////////////////////////////////////////////////
'/ Reads PORT PCF8574 
'/ Input : none
'/ Output: Return Word holds the contents of PORT
'/ Syntax: PCF8574_ReadPORT(Delay,Number Expander to read) => Delay = 0 => Interrogation Mode
'/ Notes : The ReadPORT command read the outputs bits too; to update the Shadow Bits.
'/         Saves the ports value To a shadow Variable
'/         The NACK could be checked. vPCA_ACK > 0 => error
'/  
Proc PCF8574_ReadPORT(pDelay As Byte, pNbExpander As nPCAAddress), Byte 
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_ReadPORT : START")
    '/--------------------------------------------------------------------------
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    vPCA_ACK = False                        '/ Init the error.
    F_NoACK = False
    F_SkipOverCode = False
    Result = False
    '/--------------------------------------------------------------------------
    CheckExpander()
    If vPCA_ACK = $55 Then
        'ExitProc
        GoTo Exit
    EndIf
    '/--------------------------------------------------------------------------
    If pDelay = 0 Then                      '/ Interrogation Mode.
        '/ Start Software I2C routine.
        'DelayUS 50
        GIE_Save()
        I2C_Start()                         ' Start the transmision
        I2C_WriteByte(nCTL_PCF_Read)        ' Address of the PCA9535 + WRITE
        ' Check the ACK of the device.
        If F_NoACK = True Then
            vPCA_ACK = $0F                  ' Set A NACK error
            I2C_Stop()
            PCF8574_Clear_INTxIF()
            GIE_Restore()
            'Return                         '/ Return o ExitProc is not compiled
            xGPIO = 0
            GoTo Exit
        EndIf
        xGPIO = I2C_ReadByte()              ' Read the all Inputs for the Write Procedure
        I2C_NACK()                          ' A NACK must to be sent to finish a read instruction
        I2C_Stop()                          ' Stop the transmision
        PCF8574_Clear_INTxIF()
        GIE_Restore()
        '/ End of the Software I2C routine.       
        '/ To update the outputs
        xOLAT = vIODIR[nPCAAddress]
        xOLAT = ~xGPIO & ~xOLAT             '/ State of the current ouput before change.
        vOLAT[nPCAAddress] = xOLAT          '/ Update the Outputs shadow Bits.
        '/----------------------------------------------------------------------
        '/ Read the GPIO register in Interrogation mode.
        yGPIO = xGPIO ^ vIPOL[nPCAAddress]
        yGPIO =  yGPIO & vIODIR[nPCAAddress]
        vGPIO[nPCAAddress] = yGPIO          ' The read is not from interrupt
        Result = yGPIO                      '/ Return Value
    '/--------------------------------------------------------------------------
    Else
    '/--------------------------------------------------------------------------
        DelayMS pDelay                      '/ Delay to debounce.
        '/----------------------------------------------------------------------
        '/ Third read for GPIO register after debounce Delay.
        '/ Start Software I2C routine.
        GIE_Save()
        I2C_Start()                         ' Start the transmision
        I2C_WriteByte(nCTL_PCF_Read)        ' Address of the PCF8574 + WRITE
        ' Check the ACK of the device.
        If F_NoACK = True Then
            vPCA_ACK = $0F                  ' Set A NACK error
            I2C_Stop()
            PCF8574_Clear_INTxIF()
            GIE_Restore()
            'Return                         '/ Return o ExitProc is not compiled
            xGPIO = 0
            GoTo Exit
        EndIf
        xGPIO = I2C_ReadByte()             ' Read the Byte from the PCF8574
        I2C_NACK()
        I2C_Stop()                         ' Stop the transmision
        PCF8574_Clear_INTxIF()
        GIE_Restore()
        '/ End of the Software I2C routine.      
        '/----------------------------------------------------------------------
        '/ To update the outputs
        xOLAT = vIODIR[nPCAAddress]
        xOLAT = ~xGPIO & ~xOLAT             '/ State of the current ouput before change.
        vOLAT[nPCAAddress] = xOLAT          '/ Update the Outputs shadow Bits.
        '/----------------------------------------------------------------------
        '/ Read the inputs
        xGPIO = xGPIO ^ vIPOL[nPCAAddress]
        xGPIO = xGPIO & vIODIR[nPCAAddress] 
        xINTF = xGPIO ^ vGPIO[nPCAAddress]               
        '/ The xINTF is the value of Interrupt inputs of the current expander. (For Buttons)
        vGPIO[nPCAAddress] = xGPIO 
        Result = xGPIO                

        '/ Workaround for doble interrupt
        xGPIO = xINTF & vGPIO[nPCAAddress]
        If xINTF <> xGPIO Then
            xGPIO = False                   '/ Interrupt NO valid
            xINTF = False
        EndIf
        If xGPIO = False Then
            F_SkipOverCode = True
        EndIf
   '/---------------------------------------------------------------------------
    EndIf
Exit:
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_ReadPORT : END")
    '/--------------------------------------------------------------------------
EndProc

'///////////////////////////////////////////////////////////////////////////////
'/ Writes 8 Bits To PORT Latch register PCF8574 
'/ Input  : pDataOut holds the Byte To place On the Latch
'/ Output : Returns the value of the PORT.
'/ Syntax : Return [Byte] = PCF8574_Write_PORT(DataByte, Number of Expander)
'/ Notes  : Saves the port value To vOLAT shadow Variable
'/        : vPCA_ACK could be checked.
'/
Proc PCF8574_WritePORT(pDataOut As nDataBytepcf0, pNbExpander As nPCAAddress), Byte 
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_WritePORT : START")
    '/--------------------------------------------------------------------------
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    $ifdef EncoderPinEnable
    DisableEncoder()                    '/ Disable the encoder
    $endif
    vPCA_ACK = False 
    F_NoACK = False
    vOLATOld = vOLAT[nPCAAddress]       '/ Save the Current value of PORT.
    Result = vOLATOld                   '/ If error send the Old value of PORT.
    '/--------------------------------------------------------------------------
    GIE_Save()                          '/ Disable GIE
    CheckExpander()
    If vPCA_ACK = $55 Then
        'ExitProc
        GoTo Exit
    EndIf
    '/--------------------------------------------------------------------------
    '/ Build the Output Byte.
    nDataBytepcf1 = ~nDataBytepcf0      '/ Invert the Data Out
    nDataBytepcf1 = nDataBytepcf1 | vIODIR[nPCAAddress] '/ Add inputs Bits 
    '/--------------------------------------------------------------------------
    '/ Start Software I2C routine.
    I2C_Start()                         ' Start the transmision
    I2C_WriteByte(nCTL_PCF_Write)       ' Address of the PCF8574 + WRITE
    ' Check the ACK of the device.
    If F_NoACK = True Then
        vPCA_ACK = $0F                  ' Set A NACK error
        I2C_Stop()
        'Return                         '/ Return o ExitProc are not compiled
        GoTo Exit
    EndIf
    I2C_WriteByte(nDataBytepcf1)        ' Write the Byte to the PCA9554
    I2C_Stop()                          ' Stop the transmision
    '/ Stop Software I2C routine.
    '/--------------------------------------------------------------------------
    '/ Workaround to clear the false interrupt.
    '/ Read the expander & Clear the INTx interrupt caused by writing.
    DelayUS 10
    '/ Start Software I2C routine.
    I2C_Start()                         ' Start the transmision
    I2C_WriteByte(nCTL_PCF_Read)        ' Address of the PCA9535 + WRITE
    ' Check the ACK of the device.
    If F_NoACK = True Then
        vPCA_ACK = $0F                  ' Set A NACK error
        I2C_Stop()
        'Return                         '/ Return o ExitProc are not compiled
        GoTo Exit
    EndIf
    I2C_ReadByte()                      ' Dummy Read to clear the interrupt pin.
    I2C_NACK()                          ' A NACK must to be sent to finish a read instruction
    I2C_Stop()                          ' Stop the transmision
    '/ End of the Software I2C routine.    
    '/--------------------------------------------------------------------------
    '/ The transmision is OK then load the result.
    nDataBytepcf1 = ~nDataBytepcf1
    vOLAT[nPCAAddress] = nDataBytepcf1  '/ Save the outbits of PORT into the Outputs array.
    Result = nDataBytepcf1              '/ Return Byte new outputs
    '/--------------------------------------------------------------------------
Exit:
    PCF8574_Clear_INTxIF()              '/ Clear the INTx interrupt flag in case of...
    GIE_Restore()                       '/ Restore GIE.
    $ifdef EncoderPinEnable
    EnableEncoder()                     '/ Enable the encoder for new interrupts.
    $endif
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_WritePORT : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Write To a Bit of the PORT expander (latch)
'/ Input : pPin Output Pin To Set Or Clear 
'/         pHiLo 1 = High, 0 = Low
'/ Output: OLAT To Return Var
'/ Syntax: Return [Byte] = PCA9554_WritePinPORT(pPin [Byte], HiLo [Bit])
'/ Notes : The shadow Port is updated
'/       : Testing the NACK is not mandatory but usefull.
'/ The NACK could be checked. vPCA_ACK > 0 => error
'/
Proc PCF8574_WritePinPORT(pPin As Byte, pHiLo As Bit, pNbExpander As nPCAAddress), Byte
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_WritePinPORT : START")
    '/--------------------------------------------------------------------------
    Dim ReadPORT As Byte

    #ifdef WatchDog_Req
    Clrwdt
    #endif
    Result = 0
    vPCA_ACK = False
    If nPCAAddress > PCF8574x_MaxNumber - 1 Then
        vPCA_ACK = $55
        'ExitProc
        GoTo Exit
    EndIf
    vOLATOld = vOLAT[nPCAAddress]               '/ Current value of PORT.
    Result = vOLATOld                           '/ If error send the Old value of PORT.
    '/ Now calculate the new value of the PORT
    If pPin < 8 Then                               
        nPCAtempByte1 = 1 << pPin
        If pHiLo = 1 Then
            nDataBytepcf0 = nPCAtempByte1 | vOLATOld    
        ElseIf pHiLo = 0 Then
            nPCAtempByte1 = ~nPCAtempByte1
            nDataBytepcf0 = nPCAtempByte1 & vOLATOld    
        EndIf
        Result = PCF8574_WritePORT(nDataBytepcf0, nPCAAddress)
    Else
        vPCA_ACK = $FF              '/ Error value parameter.
    EndIf
Exit:
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_WritePinPORT : END")
    '/--------------------------------------------------------------------------
EndProc

'///////////////////////////////////////////////////////////////////////////////
'/ Toggle an Output Bit of the expander PORT 16 Bits
'/ Input : Number of the Output Pin [0 to 7]
'/ Output: Return_Var vOLAT
'/ Syntax: Return [Word] OLAT = PCF8574_TogglePinPORT(Pin [Byte])
'/ Notes : Write the Pin And Refresh the OLAT shadow register.
'/         The shadow Port is updated
'/       : Testing the NACK is not mandatory but usefull.
'/ The NACK could be checked. vPCA_ACK > 0 => error
'/ vPCA_ACK = $0F => comunication error.  vPCA_ACK = $FF => error entry value.
'/
Proc PCF8574_TogglePinPORT(pPin As Byte, pNbExpander As nPCAAddress), Byte 
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_TogglePinPORT : START")
    '/--------------------------------------------------------------------------
'    Dim ReadPORT As Byte    '/ Local Byte    
    Dim NotDir As Byte      '/ Local Byte

    #ifdef WatchDog_Req
    Clrwdt
    #endif
    vPCA_ACK = False
    Result = 0
    If nPCAAddress > PCF8574x_MaxNumber - 1 Then
        vPCA_ACK = $55
        'ExitProc
        GoTo Exit
    EndIf
    vOLATOld = vOLAT[nPCAAddress]                '/ Save the Current value of PORT.
    Result = vOLATOld                            '/ If error send the Old value of PORT.
    '/ Now calculate the new value of the PORT
    If pPin < 8 Then
        nPCAtempByte1 = 1 << pPin
        NotDir = vIODIR[nPCAAddress]            '/ Congig IODIR of this PORT.
        NotDir = ~NotDir
        nPCAtempByte1 = nPCAtempByte1 & NotDir  ' Mask the outputs bits (outputs only)
        nDataBytepcf0 = vOLATOld ^ nPCAtempByte1
        Result = PCF8574_WritePORT(nDataBytepcf0, nPCAAddress)
    Else
        vPCA_ACK = $FF                          '/ Error value parameter.    
    EndIf
Exit:
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_TogglePinPORT : END")
    '/--------------------------------------------------------------------------
EndProc

'///////////////////////////////////////////////////////////////////////////////
'/ Initialize the PCAPCF8574 8 bits EXPANDER
'/ Syntax    : CheckAck = PCF8574_Init()
'/ Return    : vPCA_ACK
'/
Proc PCF8574_Init(),vPCA_ACK 
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_Init : START")
    '/--------------------------------------------------------------------------
    $if PCF8574x_MaxNumber > 0
        #ifndef IODIR0_Configured
            #error "The IODIR0 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 1
        #ifndef IODIR1_Configured
            #error "The IODIR1 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 2
        #ifndef IODIR2_Configured
            #error "The IODIR2 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 3
        #ifndef IODIR3_Configured
            #error "The IODIR3 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 4
        #ifndef IODIR4_Configured
            #error "The IODIR4 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 5
        #ifndef IODIR5_Configured
            #error "The IODIR5 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 6
        #ifndef IODIR6_Configured
            #error "The IODIR6 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 7
        #ifndef IODIR7_Configured
            #error "The IODIR7 is NOT declared!"
        #endif
    $endif
    '/--------------------------------------------------------------------------
    $if PCF8574x_MaxNumber > 0
        #ifndef IPOL0_Configured
            #error "The IPOL0 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 1
        #ifndef IPOL1_Configured
            #error "The IPOL1 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 2
        #ifndef IPOL2_Configured
            #error "The IPOL2 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 3
        #ifndef IPOL3_Configured
            #error "The IPOL2 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 4
        #ifndef IPOL4_Configured
            #error "The IPOL4 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 5
        #ifndef IPOL5_Configured
            #error "The IPOL5 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 6
        #ifndef IPOL6_Configured
            #error "The IPOL6 is NOT declared!"
        #endif
    $endif
    $if PCF8574x_MaxNumber > 7
        #ifndef IPOL7_Configured
            #error "The IPOL7 is NOT declared!"
        #endif
    $endif
    '/--------------------------------------------------------------------------
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    $ifdef EncoderPinEnable
    Input EncoderPin0
    Input EncoderPin1    
    Input EncoderPin2
    DisableEncoder()                   '/ Disable the encoder
    $endif
    
    PCA_Flags = False
    vPCA_ACK = False                    ' Clear the ACK error flag.

    ' Setup the INTx of PORTB.x interrupt.
    #ifdef __INTERRUPTS_ENABLED
        INTCONBits_GIE = False
        $ifdef _INT0_
            $ifdef _ANSELB
            ANSELB.0 = False    ' PORTB.0 as digital
            $endif
            $ifdef _ANSELH
            ANSELH.4 = False
            $endif
            Input PORTB.0       ' For Interrupt input
            INTEDG0 = False     ' 0 = Interrupt on falling edge
            INT0IE = False      ' 0 = INT0 External Interrupt Enable bit
        $endif
        $ifdef _INT1_
            $ifdef _ANSELB
            ANSELB.1 = False    ' PORTB.1 as digital
            $endif
            $ifdef _ANSELH
            ANSELH.2 = False
            $endif
            Input PORTB.1       ' For Interrupt input
            INTEDG1 = False     ' 0 = Interrupt on falling edge
            INT1IE = False      ' 0 = INT1 External Interrupt Enable bit
            INT1IP = True       ' 1 = High External Interrupt Priority 
        $endif        
        $ifdef _INT2_
            $ifdef _ANSELB
            ANSELB.2 = False    ' PORTB.2 as digital
            $endif
            $ifdef _ANSELH
            ANSELH.0 = False
            $endif
            Input PORTB.2       ' For Interrupt input
            INTEDG2 = False     ' 0 = Interrupt on falling edge
            INT2IE = False      ' 0 = INT2 External Interrupt Enable
            INT2IP = True       ' 1 = High External Interrupt Priority
        $endif
    #endif
    '---------------------------------------------------------------------------
    '/ Clear all Expanders
    For nPCAAddress = 0 To (PCF8574x_MaxNumber - 1) Step 1
        PCF8574_WritePORT($00, nPCAAddress)
        If vPCA_ACK = $0F Then
            '/ nPCAAddress is the Number of expander in fault. 
            GoTo Exit
        EndIf
        DelayUS 10
    Next nPCAAddress
    '---------------------------------------------------------------------------
    '/ Read all Expanders
    For nPCAAddress = 0 To (PCF8574x_MaxNumber - 1) Step 1
        vGPIO[nPCAAddress] = PCF8574_ReadPORT(0, nPCAAddress)
        If vPCA_ACK = $0F Then
            '/ nPCAAddress is the Number of expander in fault.
            GoTo Exit
        EndIf
        DelayUS 10
    Next nPCAAddress
    '---------------------------------------------------------------------------
    ' Setup the INTx of PORTB.x interrupt.
    PCF8574_Enable_INTxIE()   
    INTCONBits_GIE = True     
    INTCONBits_PEIE = True   
    '/--------------------------------------------------------------------------
    $ifdef EncoderPinEnable
    EnableEncoder()                   '/ Enable the encoder
    $endif
Exit:
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: PCF8574_Init : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
'/ Check the INTx Pins of the PIC.
'/ Input : none
'/ Output: Terminal
'/ Syntax: CheckINTxPins()
'/ Notes : Compiled only if the UART is Declared.
'/
Proc CheckINTxPins()
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: CheckINTxPins : START")
    '/--------------------------------------------------------------------------
    Dim Temp As Byte    '/ Local variable

#ifdef Hserial_TXSTA

    #ifdef WatchDog_Req
    Clrwdt
    #endif
    Temp = PORTB
    $ifdef _INT0_
    If Temp.0 = True Then
        HRSOut "INT0 Pin is High",CR
    Else
        HRSOut "INT0 Pin is Low",CR
    EndIf
    $endif

    $ifdef _INT1_
    If Temp.1 = True Then
        HRSOut "INT1 Pin is High",CR
    Else
        HRSOut "INT1 Pin is Low",CR
    EndIf
    $endif            

    $ifdef _INT2_
    If Temp.2 = True Then
        HRSOut "INT2 Pin is High",CR
    Else
        HRSOut "INT2 Pin is Low",CR
    EndIf
    $endif 
#endif
    '/--------------------------------------------------------------------------
    WriteMSG("PROCEDURE: CheckINTxPins : END")
    '/--------------------------------------------------------------------------
EndProc
'///////////////////////////////////////////////////////////////////////////////
PCF_OverDriver:
'/ Initialisation of the I2C Bus (Mandatory)
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    Low _SDA_Pin
    Low _SCL_Pin
    DelayUS 10
    Input _SDA_Pin
    Input _SCL_Pin
    DelayUS 10
    I2C_Stop() 
'///////////////////////////////////////////////////////////////////////////////
$endif      ' _PCF8474_



